@@ -9,7 +9,7 @@
 #' @param tokenizer_parallelism (boolean)  If TRUE this will turn on tokenizer parallelism.
 #' @param logging_level (string)  Set the logging level.
 #' Options (ordered from less logging to more logging): critical, error, warning, info, debug
-#' @param return_incorrect_results (boolean)  Stop returning some incorrectly formatted/structured results.
+#' @param force_return_results (boolean)  Stop returning some incorrectly formatted/structured results.
 #' This setting does CANOT evaluate the actual results (whether or not they make sense, exist, etc.).
 #' All it does is to ensure the returned results are formatted correctly (e.g., does the question-answering
 #' dictionary contain the key "answer", is sentiments from textClassify containing the labels "positive"
@@ -37,7 +37,7 @@ textSum <- function(x,
                     device = "cpu",
                     tokenizer_parallelism = FALSE,
                     logging_level = "warning",
-                    return_incorrect_results = FALSE,
+                    force_return_results = FALSE,
                     return_text = TRUE,
                     return_tensors = FALSE,
                     clean_up_tokenization_spaces = FALSE,
@@ -66,7 +66,7 @@ textSum <- function(x,
       device = device,
       tokenizer_parallelism = tokenizer_parallelism,
       logging_level = logging_level,
-      return_incorrect_results = return_incorrect_results,
+      force_return_results = force_return_results,
       return_text = return_text,
       return_tensors = return_tensors,
       clean_up_tokenization_spaces = clean_up_tokenization_spaces,
@@ -100,7 +100,7 @@ textSum <- function(x,
       sep = " "
     )
 
-    cat(colourise(loop_text, "green"))
+    message(colourise(loop_text, "green"))
   }
 
   ALL_output1 <- dplyr::bind_cols(ALL_output)
