@@ -20,11 +20,12 @@ test_that("PM tutorial", {
   # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 
   # Transform the text/word data to word embeddings (see help(textEmbed) to see the default settings).
-  word_embeddings <- text::textEmbed(Language_based_assessment_data_8,
-                               model = "bert-base-uncased",
-                               aggregation_from_layers_to_tokens = "concatenate",
-                               aggregation_from_tokens_to_texts = "mean",
-                               keep_token_embeddings = FALSE)
+  word_embeddings <- text::textEmbed(
+    Language_based_assessment_data_8,
+    model = "bert-base-uncased",
+    aggregation_from_layers_to_tokens = "concatenate",
+    aggregation_from_tokens_to_texts = "mean",
+    keep_token_embeddings = FALSE)
 
   # See how the word embeddings are structured
   #word_embeddings
@@ -195,7 +196,7 @@ test_that("PM tutorial", {
 
   projection_results$word_data
   expect_equal(projection_results$word_data[[1]][[1]], "Group1*")
-  expect_equal(projection_results$word_data$dot.x[[1]], -14.87054, tolerance = 0.00001)
+  expect_equal(projection_results$word_data$dot.x[[1]], -15.11895, tolerance = 0.00001)
   # To avoid warnings -- and that words do not get plotted, first increase the max.overlaps for the entire session:
   #options(ggrepel.max.overlaps = 1000)
 
